#------------------------------------------------------------------------------
#    Copyright (C) 2009 Florian Schanda <florian@schanda.org.uk>
#
#    This file is part of the Warzone Map Tools.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#------------------------------------------------------------------------------

cdef extern from "stdlib.h":
    void *malloc(int size)
    void free(void *ptr)

def blur(hmap, int radius, exclude, int weight):
    print "Blurring with radius %u..." % radius
    cdef int sx, sy
    cdef int x, y
    cdef int i, j
    cdef int n, total

    sx, sy = max(hmap.iterkeys())
    sx += 1
    sy += 1
    cdef int *src = <int*>malloc(sx*sy*sizeof(int))
    cdef int *dst = <int*>malloc(sx*sy*sizeof(int))
    cdef int *exc = <int*>malloc(sx*sy*sizeof(int))
    for y in xrange(sy):
        for x in xrange(sx):
            src[x + sx*y] = hmap[(x, y)]
            exc[x + sx*y] = 1 if (x, y) in exclude else 0
    
    for y in xrange(sy):
        for x in xrange(sx):
            if exc[x + sx*y] == 1:
                dst[x + sx*y] = src[x + sx*y]
                continue

            total = weight * src[x + sx*y]
            n = weight
            for j in xrange(y-radius, y+radius+1):
                if j < 0 or j >= sy:
                    continue
                for i in xrange(x-radius, x+radius+1):
                    if i < 0 or i >= sx:
                        continue
                    # Make a circle
                    if (i-x)*(i-x) + (j-y)*(j-y) > radius*radius:
                        continue
                    total += src[i + sx*j]
                    n += 1
            dst[x + sx*y] = total / n
            #print n, src[x + sx*y], dst[x + sx*y]

    for y in xrange(sy):
        for x in xrange(sx):
            hmap[(x, y)] = dst[x + sx*y]
    
    free(<void*>src)
    free(<void*>dst)
    free(<void*>exc)
