#!/usr/bin/env python
#------------------------------------------------------------------------------
#    Copyright (C) 2009 Florian Schanda <florian@schanda.org.uk>
#
#    This file is part of the Warzone Map Tools.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#------------------------------------------------------------------------------

import os
import sys
from random import randint, choice
from optparse import OptionParser

from gen_cliff import cliffmap
from status_update import StatusUpdate
from mapwrite import install_map

parser = OptionParser()
parser.add_option("--name",
                  dest="mapname", default="MakeMap", action="store",
                  help="Name of the map.")
parser.add_option("--players",
                  dest="players", default=4, type="int", action="store",
                  help="Number of players (can be 2, 4 or 8).")
parser.add_option("--size",
                  dest="size", action="store", default=None,
                  help="Map size (can be small, medium, large).")
parser.add_option("--version",
                  dest="version", action="store", default="2.2",
                  help="Warzone version (can be 2.2 or trunk)")

options, args = parser.parse_args()

if options.size is None:
    options.size = choice(["small", "medium", "large"])

if options.players not in (2, 4, 8):
    print "Players must be 2, 4 or 8."
    sys.exit(1)

if options.size not in ("small", "medium", "large"):
    print "Map size must be 'small', 'medium' or 'large'."
    sys.exit(1)

if options.version not in ("2.2", "trunk"):
    print "Warzone version must be either '2.2' or 'trunk'."
    sys.exit(1)

attr = {
    "wzversion": options.version,
    "tileset": "Random",
    "depth": randint(4, 7),
    "players": options.players,
    "baseoil": randint(4, 6),
    "freeoil_clusters": options.players // 2,
    "freeoil_style": choice([(1, 1), (1, 2), (2, 3), (3, 4), (4, 4), (1, 4)]),
    "freeoil_clustersize": randint(1, 2),
    "emptymiddle": 0,
    "everythingreachable": randint(0, 1),
    "basestyle": (2, 2, 2, 2),
    "baseheight": choice([(0, 0), (1, 0), (0, 1)]),
    "asp_bases": True,
    "scale": 8,
    "global_feature": "none",
    }

schema = randint(0, 7)
if schema == 0:
    attr["waterdensity"] = 0 if randint(0, 1) else randint(5, 10)
    attr["causeways"] = 0
    attr["density"] = randint(15, 75)
elif schema == 1:
    attr["waterdensity"] = 0
    attr["causeways"] = choice([3, 60])
    attr["density"] = randint(35, 55)
elif schema == 2:
    attr["waterdensity"] = randint(20, 30)
    attr["causeways"] = 0
    attr["density"] = randint(15, 35)
else:
    attr["waterdensity"] = 0 if randint(0, 1) else randint(5, 10)
    attr["causeways"] = 0
    attr["density"] = randint(45, 55)
    if attr["players"] == 8:
        attr["global_feature"] = choice(["wasteland", "lake", "continent"])
    else:
        attr["global_feature"] = choice(["hotspot", "wasteland", "lake",
                                         "continent"])
    

MIN_CELLS = {
    2: 2*10,
    4: 4*11,
    8: 100,
    }

if options.size == "small":
    attr["mapstyle"] = "random"
    attr["freeoil_clusters"] += 0
    if options.players == 8:
        attr["causeways"] = 0
        attr["waterdensity"] = 0
    while True:
        attr["sx"] = randint(8, 16)
        attr["sy"] = randint(8, 16)
        if attr["sx"] > attr["sy"]:
            continue
        if attr["sx"] * attr["sy"] <= MIN_CELLS[options.players]:
            continue
        if attr["sx"] * attr["sy"] >= options.players*8 + 60:
            continue
        break

    while (attr["sx"]*attr["scale"] * attr["sy"]*attr["scale"]) <= 85**2:
        attr["scale"] += 1
elif options.size == "medium" and randint(0, 1):
    attr["mapstyle"] = "random"
    attr["freeoil_clusters"] += randint(0, 2)
    if options.players == 8:
        attr["causeways"] = 0
    while True:
        attr["sx"] = randint(9, 16)
        attr["sy"] = randint(9, 16)
        if attr["sx"] > attr["sy"]:
            continue
        if attr["sx"] * attr["sy"] <= int(MIN_CELLS[options.players] *
                                          (100.0+attr["waterdensity"])/100.0):
            continue
        if attr["sx"] * attr["sy"] >= options.players*8 + 100:
            continue
        break

    while (attr["sx"]*attr["scale"] * attr["sy"]*attr["scale"]) <= 130**2:
        attr["scale"] += 1
elif options.size == "medium":
    attr["mapstyle"] = "symmetry"
    attr["freeoil_clusters"] += randint(1, 3)
    while True:
        attr["sx"] = randint(7, 10)
        attr["sy"] = attr["sx"]
        rs = (attr["sx"]*2 - 1) * (attr["sy"]*2 - 1)
        if rs <= MIN_CELLS[options.players]:
            continue
        break

    while (rs*attr["scale"]*attr["scale"]) <= 130**2:
        attr["scale"] += 1
elif options.size == "large":
    attr["mapstyle"] = "symmetry"
    attr["freeoil_clusters"] += randint(2, 4)
    attr["sx"] = randint(8, 12)
    attr["sy"] = attr["sx"]
    rs = (attr["sx"]*2 - 1) * (attr["sy"]*2 - 1)

    while (rs*attr["scale"]*attr["scale"]) <= 230**2:
        attr["scale"] += 1
        
print "Creating [%s] with the following options:" % options.mapname
for a in attr:
    print "\t%s = %s" % (a, attr[a])

for tp in cliffmap(attr):
    if isinstance(tp, StatusUpdate):
        print "\033[32m***", tp.text, "***\033[0m"
    else:
        install_map(tp.save(options.mapname))
        sys.exit(0)

sys.exit(1)
