// 
// Copyright (c) 2006-2007, Benjamin Kaufmann
// 
// This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
// 
// Clasp is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Clasp is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Clasp; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef CLASP_TIMER_H_INCLUDED
#define CLASP_TIMER_H_INCLUDED

#ifdef _MSC_VER
#pragma once
#endif
#include <clasp/include/util/platform.h> // uint64

namespace Clasp {

class Timer {
public:
	Timer() : start_(0), split_(0), total_(0) {}
	void   start()   { start_ = clockStamp(); }
	void   stop()    { split_ = clockStamp() - start_; total_ += split_; }
	void   reset()   { *this  = Timer(); }
	//! elapsed time (in seconds) for last start-stop cycle
	double elapsed() const { 
		return split_ / double(ticksPerSec());
	}
	//! total elapsed time for all start-stop cycles 
	double total()   const {
		return total_ / double(ticksPerSec());
	}
	static uint64 clockStamp();
	static uint64 ticksPerSec();
private:
	uint64 start_;
	uint64 split_;
	uint64 total_;
};

}
#endif
