/*****************************************************************************
 *  Copyright (C) 2009 Florian Schanda <florian@schanda.org.uk>              *
 *                                                                           *
 *  This file is part of the Warzone Map Tools.                              *
 *                                                                           *
 *  This program is free software: you can redistribute it and/or modify     *
 *  it under the terms of the GNU General Public License as published by     *
 *  the Free Software Foundation, either version 3 of the License, or        *
 *  (at your option) any later version.                                      *
 *                                                                           *
 *  This program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
 *  GNU General Public License for more details.                             *
 *                                                                           *
 *  You should have received a copy of the GNU General Public License        *
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.     *
 *****************************************************************************/

#include <stdlib.h>
#include "sorted_list.h"

SortedList * new_slist(void)
{
  SortedList * rv = malloc(sizeof(SortedList));
  rv->head = NULL;
  return rv;
}

Node * new_node(int coord, int cost, Node *next)
{
  Node * rv = malloc(sizeof(Node));
  rv->coord = coord;
  rv->cost = cost;
  rv->next = next;
  return rv;
}

void delete_slist(SortedList *l)
{
  Node * p = l->head;
  while (p) {
	Node * t = p;
	p = p->next;
	free(t);
  }
  free(l);
}

void slist_add(SortedList *l, int coord, int cost)
{
  Node * p = l->head;
  Node * prev = NULL;

  // Add
  if (p == NULL) {
	l->head = new_node(coord, cost, l->head);
	return;
  }
  while (p) {
	if (cost <= p->cost) {
	  if (prev) {
		prev->next = new_node(coord, cost, p);
		prev = prev->next;
	  } else {
		l->head = new_node(coord, cost, p);
		prev = l->head;
	  }
	  break;
	} else if (p->coord == coord) {
	  return;
	}
	prev = p;
	p = p->next;
  }
  if (p == NULL) {
	prev->next = new_node(coord, cost, NULL);
	return;
  }
	  
  // Purge
  while (p) {
	if (p->coord == coord) {
	  prev->next = p->next;
	  free(p);
	  p = prev->next;
	  return;
	}
	prev = p;
	p = p->next;
  }
}
  
void slist_pop(SortedList *l)
{
  l->head = l->head->next;
}

int slist_head_coord(SortedList *l)
{
  return l->head->coord;
}

int slist_head_cost(SortedList *l)
{
  return l->head->cost;
}

int slist_is_empty(SortedList *l)
{
  return l->head == NULL;
}
