#!/usr/bin/env python
#------------------------------------------------------------------------------
#    Copyright (C) 2009 Florian Schanda <florian@schanda.org.uk>
#
#    This file is part of the Warzone Map Tools.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#------------------------------------------------------------------------------

from tileset import TileSet

class Arizona(TileSet):
    def __init__(self):
        TileSet.__init__(self)

        self.name = "Arizona"
        self.terrains = [
            "Yellow Dirt",
            "Yellow Dirt Shrubs",
            "Red Dirt",
            "Red Clay",
            "Dark Clay",
            "Tiles",
            ]
        
        self.terraintiles = {
            "Yellow Dirt": [(12, 20), (55, 1)],
            "Yellow Dirt Shrubs" : [(0, 1), (9, 1), (11, 1)],
            "Red Dirt": [(44, 10), (76, 10), (48, 8), (53, 7), (54, 8),
                         (74, 6), (56, 2)],
            "Red Clay": [(5, 1), (6, 1), (7, 1), (8, 1)],
            "Dark Clay": [(23, 20), (62, 1)],
            "Tiles": [(22, 1), (77, 1)],
            "Water": [(17, 1)]
            }

        self.cliff_x = 18

        self.transtiles = {
            # Standard transitions.
            # 0 = base
            # 1 = the new tile
            ("Red Dirt", "Red Clay"): {
                (0, 1,
                 0, 1) : 34,
                (0, 0,
                 0, 1) : 35,
                (0, 1,
                 1, 1) : 36                 
                },
            ("Red Dirt", "Yellow Dirt Shrubs"): {
                (1, 0,
                 1, 0) : 27,
                (1, 1,
                 1, 0) : 28,
                (1, 0,
                 0, 0) : 29
                },
            ("Red Dirt", "Tiles"): {
                (1, 0,
                 1, 0) : 21,
                (1, 1,
                 1, 0) : 19,
                (1, 0,
                 0, 0) : 20
                },
            ("Red Dirt", "Yellow Dirt"): {
                (0, 1,
                 0, 1) : 43,
                (0, 1,
                 1, 1) : 42,
                (0, 0,
                 0, 1) : 41                   
                },
            ("Yellow Dirt", "Red Clay"): {
                (0, 1,
                 0, 1) : 2,
                (1, 1,
                 1, 0) : 3,
                (1, 0,
                 0, 0) : 4
                },
            ("Red Clay", "Dark Clay"): {
                (1, 0,
                 1, 0) : 24,
                (1, 1,
                 1, 0) : 25,
                (1, 0,
                 0, 0) : 26
                },

            ("Yellow Dirt", "Water"): {
                (1, 1,
                 0, 0) : 14,
                (0, 1,
                 0, 0) : 15,
                (1, 1,
                 0, 1) : 16
                },
            ("Dark Clay", "Water"): {
                (0, 0,
                 1, 1) : 31,
                (0, 0,
                 0, 1) : 32,
                (0, 1,
                 1, 1) : 33
                },
            
            # Cliffs
            ("Red Dirt", "Cliff"): {
                (1, 1,
                 0, 0) : 71,
                (0, 1,
                 1, 1) : 75,
                (1, 0,
                 0, 0) : 75
                },

            # Roads
            ("Red Dirt", "Track"): {
                (  1,
                 0,  0,
                   1)   : (49, 51),
                (  0,
                 0,  1,
                   1)   : 50,
                (  0,
                 0,  0,
                   1)   : 52,
                (  1,
                 0,  1,
                   1)   : 72,
                (  1,
                 1,  1,
                   1)   : 73,
                },

            }

        self.feat_buildings = {
            "Ruin1" : {"size": (1, 1)},
            "miruin2" : {"size": (1, 1)},
            "Ruin3" : {"size": (1, 1)},
            "Ruin4" : {"size": (1, 1)},
            "Ruin5" : {"size": (1, 1)},
            "BarbHUT" : {"size": (1, 1)},
            "Pylon" : {"size": (1, 1)},
            }

        self.feat_techbuildings = {
            "BarbTechRuin" : {"size": (2, 1)},
            "BarbWarehouse1" : {"size": (2, 1)},
            "BarbWarehouse2" : {"size": (2, 1)},
            "BarbWarehouse3" : {"size": (1, 1)},
            "WaterTower" : {"size": (1, 1)},
            "Crane" : {"size": (1, 1)},
            "OilTower" : {"size": (1, 1)},
            "OldFactory" : {"size": (2, 2)},
            "AirTrafficControl" : {"size": (1, 1)},
            "OilDrum" : {"size": (1, 1)},
            }

        self.feat_cars = {
            "WreckedTankerV" : {"size": (1, 1)},
            "WreckedSuzukiJeep" : {"size": (1, 1)},
            "WreckedVertCampVan" : {"size": (1, 1)},
            }

        self.feat_rocks = {
            "Boulder1" : {"size": (1, 1)},
            "Boulder2" : {"size": (1, 1)},
            "Boulder3" : {"size": (1, 1)},
            }
