#!/usr/bin/env python
#------------------------------------------------------------------------------
#    Copyright (C) 2009 Florian Schanda <florian@schanda.org.uk>
#
#    This file is part of the Warzone Map Tools.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#------------------------------------------------------------------------------

from tileset import TileSet

class Rockies(TileSet):
    def __init__(self):
        TileSet.__init__(self)

        self.name = "Rockies"
        self.terrains = [
            "Grass",
            "Grey Dirt",
            "Brown Dirt",
            "Tiles",
            "Dirt Snow",
            "Grass Snow",
            "Snow"
            ]
        
        self.terraintiles = {
            "Grass": [(0, 20), (1, 3), (47, 2), (74, 2), (58, 1)],
            "Grey Dirt": [(5, 15), (6, 15), (7, 15),
                          (8, 2), (43, 2), (55, 3), (57, 3), (62, 1)],
            "Brown Dirt": [(27, 5), (28, 5), (53, 15), (56, 1)],
            "Tiles": [(22, 30), (79, 1)],
            "Dirt Snow": [(41, 25), (73, 2), (75, 2)],
            "Grass Snow": [(23, 1)],
            "Snow": [(64, 20), (70, 1)],
            "Water": [(17, 1)]
            }

        self.cliff_x = 30

        self.transtiles = {
            # Standard transitions.
            # 0 = base
            # 1 = the new tile
            ("Grass", "Grey Dirt") : {
                (1, 0,
                 1, 0) : 2,
                (1, 0,
                 0, 0) : 3,
                (1, 1,
                 1, 0) : 4
                },
            ("Grey Dirt", "Dirt Snow") : {
                (1, 1,
                 0, 0) : 12,
                (1, 0,
                 0, 0) : 11,
                (1, 1,
                 1, 0) : 10
                },
            ("Brown Dirt", "Tiles") : {
                (1, 1,
                 1, 0) : 19,
                (1, 0,
                 0, 0) : 20,
                (1, 0,
                 1, 0) : 21
                },
            ("Grass", "Grass Snow") : {
                (1, 1,
                 1, 0) : 24,
                (1, 0,
                 0, 0) : 25,
                (1, 1,
                 0, 0) : 26
                },
            ("Grass", "Brown Dirt") : {
                (0, 1,
                 0, 1) : 34,
                (0, 0,
                 0, 1) : 35,
                (0, 1,
                 1, 1) : 36
                },
            ("Brown Dirt", "Grey Dirt") : {
                (0, 1,
                 0, 1) : 38,
                (0, 0,
                 0, 1) : 39,
                (0, 1,
                 1, 1) : 40
                },
            ("Dirt Snow", "Snow") : {
                (1, 1,
                 1, 0) : 65,
                (1, 0,
                 0, 0) : 66,
                (1, 1,
                 0, 0) : 67
                },

            # Water
            ("Grass", "Water") : {
                (1, 1,
                 0, 0) : 14,
                (0, 1,
                 0, 0) : 15,
                (1, 1,
                 0, 1) : 16
                },
            ("Grey Dirt", "Water") : {
                (0, 0,
                 1, 1) : 31,
                (0, 1,
                 1, 1) : 32,
                (0, 0,
                 0, 1) : 33
                },

            # Cliffs
            ("Dirt Snow", "Cliff"): {
                (1, 1,
                 0, 0) : 44,
                (0, 1,
                 1, 1) : 9,
                (1, 0,
                 0, 0) : 9
                },
            ("Grey Dirt", "Cliff"): {
                (1, 1,
                 0, 0) : 46,
                (0, 1,
                 0, 0) : 45,
                (1, 0,
                 1, 1) : 45
                },

            # Roads
            ("Brown Dirt", "Track"): {
                (  1,
                 0,  0,
                   1)   : (49, 51),
                (  0,
                 0,  1,
                   1)   : 50,
                (  0,
                 0,  0,
                   1)   : 52,
                (  1,
                 0,  1,
                   1)   : 72
                },

            }

        self.feat_buildings = {
            "LogCabin1" : {"size": (1, 1)},
            "LogCabin2" : {"size": (1, 1)},
            "LogCabin3" : {"size": (1, 1)},
            "LogCabin4" : {"size": (1, 1)},
            "LogCabin5" : {"size": (1, 1)},
            }

        self.feat_misc = {
            "Tree1" : {"size": (2, 2)},
            "Tree2" : {"size": (2, 2)},
            "Tree3" : {"size": (1, 1)},
            "TreeSnow1" : {"size": (2, 2)},
            "TreeSnow2" : {"size": (2, 2)},
            "TreeSnow3" : {"size": (1, 1)},
            }
