#!/usr/bin/env python
#------------------------------------------------------------------------------
#    Copyright (C) 2009 Florian Schanda <florian@schanda.org.uk>
#
#    This file is part of the Warzone Map Tools.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#------------------------------------------------------------------------------

from tileset import TileSet

class Urban(TileSet):
    def __init__(self):
        TileSet.__init__(self)

        self.name = "Urban"
        self.terrains = [
            "Dark Ground",
            "Light Debris",
            "Tiles",
            "Dark Tiles",
            "Brown Rocks",
            "Grass"
            ]
        
        self.terraintiles = {
            "Dark Ground": [(0, 20), (1, 2), (2, 5), (32, 1), (57, 1)],
            "Light Debris": [(5, 10), (7, 10), (8, 10), (55, 1), (78, 1)],
            "Tiles": [(20, 2), (21, 4), (28, 1), (48, 1),
                      (51, 35), (52, 4), (59, 4), (61, 1), (74, 1)],
            "Dark Tiles": [(18, 1)],
            "Brown Rocks": [(22, 20), (31, 5), (36, 2)],
            "Grass" : [(50, 1)],
            "Water": [(17, 1)],
            }

        self.transtiles = {
            # Standard transitions.
            # 0 = base
            # 1 = the new tile
            ("Dark Ground", "Light Debris"): {
                (0, 1,
                 0, 1) : 6,
                (1, 1,
                 1, 0) : 4,
                (1, 0,
                 0, 0) : 3
                },
            ("Dark Tiles", "Tiles"): {
                (1, 1,
                 0, 1) : 11,
                (0, 1,
                 0, 1) : 12,
                (0, 0,
                 1, 0) : 18
                },
            ("Light Debris", "Tiles"): {
                (1, 0,
                 1, 0) : 27,
                (0, 1,
                 0, 0) : 29,
                (1, 1,
                 0, 1) : 30
                },
            ("Dark Ground", "Brown Rocks"): {
                (1, 0,
                 1, 0) : 33,
                (1, 1,
                 1, 0) : 34,
                (1, 0,
                 0, 0) : 35
                },
            ("Grass", "Brown Rocks"): {
                (0, 0,
                 0, 1) : 37,
                (0, 1,
                 1, 1) : 38,
                (1, 0,
                 1, 0) : 39
                },
            ("Light Debris", "Brown Rocks"): {
                (0, 0,
                 0, 1) : 72,
                (0, 1,
                 1, 1) : 73,
                (1, 0,
                 1, 0) : 60
                },
            ("Brown Rocks", "Tiles"): {
                (0, 1,
                 0, 1) : 71,
                (0, 1,
                 1, 1) : 75,
                (0, 0,
                 0, 1) : 76
                },
            ("Grass", "Light Debris"): {
                (1, 0,
                 1, 0) : 77,
                (0, 1,
                 0, 0) : 79,
                (1, 1,
                 0, 1) : 58
                },
 
            ("Tiles", "Water"): {
                (1, 1,
                 0, 0) : 14,
                (0, 1,
                 0, 0) : 15,
                (1, 1,
                 0, 1) : 16
                },
            ("Light Debris", "Water"): {
                (0, 0,
                 1, 1) : 24,
                (0, 1,
                 1, 1) : 25,
                (0, 0,
                 0, 1) : 26
                },
            
            # Cliffs
            ("Light Debris", "Cliff"): {
                (1, 1,
                 0, 0) : (69, 70),
                (0, 1,
                 0, 0) : 68,
                (1, 0,
                 1, 1) : 68
                },

            # Road
            (None, "Road"): {
                (  0,
                 1,  1,
                   0)   : (42,)*10 + (46, 47, 53),
                (  0,
                 1,  1,
                   1)   : 40,
                },

            ("Light Debris", "Road"): {
                (  0,
                 1,  0,
                   0)   : (43, 44),
                },

            ("Dark Ground", "Road"): {
                (  0,
                 1,  0,
                   0)   : 41
                },

            ("Grass", "Road"): {
                (  0,
                 1,  0,
                   0)   : 45
                },

            }
        
        self.feat_buildings = {
            "building1" : {"size": (3, 3)},
            "building2" : {"size": (3, 3)},
            "building3" : {"size": (3, 3)},
            "building7" : {"size": (3, 3)},
            "building8" : {"size": (3, 3)},
            "building11": {"size": (3, 3)},
            "WreckedBuilding16": {"size": (3, 3)},
            "building10": {"size": (2, 1)},
            "building12": {"size": (2, 1)},
            "WreckedBuilding9": {"size": (2, 2)},
            "WreckedBuilding17": {"size": (2, 2)},
            #"OldFactory": {"size": (2, 2)},
            "Crane": {"size": (1, 1)},
            #"OilTower": {"size": (1, 1)},
            "Pylon": {"size": (1, 1)},
            #"AirTrafficControl": {"size": (1, 1)},
            }

        self.feat_road = {
            #"WreckedDroidHub" : {"size": (1, 1)},
            "WreckedVertCampVan" : {"size": (1, 1)},
            "WreckedSuzukiJeep" : {"size": (1, 1)},
            "Chevy" : {"size": (1, 1)},
            "BlueCar" : {"size": (1, 1)},
            "Pickup" : {"size": (1, 1)},
            }
